#!/usr/bin/env python3

# ---------------------------------------------------------------------
# Install python3-pymrpt, ros-$ROS_DISTRO-python-mrpt,
# ros-$ROS_DISTRO-mrpt2, or test with a local build with:
# export PYTHONPATH=$HOME/code/mrpt/build-Release/:$PYTHONPATH
# ---------------------------------------------------------------------

# More matrix classes available in the module mrpt.math.
# See: https://mrpt.github.io/pymrpt-docs/mrpt.pymrpt.mrpt.math.html

from mrpt.pymrpt import mrpt
import numpy as np

# Create a numpy matrix from a list:
m1_np = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])

print('m1_np   :\n' + str(m1_np))
print()

# Create an MRPT matrix from a list:
m1_mrpt = mrpt.math.CMatrixDynamic_double_t(
    [[11, 12, 13], [14, 15, 16], [17, 18, 19]])

print('m1_mrpt :\n' + str(m1_mrpt))
print('m1_mrpt.size() :' + str(m1_mrpt.size()))
print()

# Convert an MRPT matrix to numpy via an intermediary list:
m2_mrpt = mrpt.math.CMatrixDynamic_double_t.Identity(3)
m2_data = m2_mrpt.to_list()
print('m2_data: {}'.format(m2_data))
m2_np = np.array(m2_data)
print('m2_np:\n{}'.format(m2_np))
print()

# Read/write access an MRPT matrix (0-based indices)
m2_mrpt[0, 2] = 99.0  # modify the entry (0,2)
print('m2 modified:\n{}'.format(m2_mrpt))
m2_mrpt[0, 2] = 99.0  # modify the entry (0,2)
print('m2[1,1]={}'.format(m2_mrpt[1, 1]))
print()

# Convert a numpy matrix to MRPT:
m3_np = np.array([[1, 2], [3, 4]])
m3_mrpt = mrpt.math.CMatrixDynamic_double_t(m3_np.tolist())
print('m3_np:\n{}'.format(m3_np))
print('m3_mrpt:\n{}'.format(m3_mrpt))



from typing import Any, ClassVar

from typing import overload
import mrpt.pymrpt.mrpt.img
import mrpt.pymrpt.mrpt.opengl
import mrpt.pymrpt.mrpt.system
import mrpt.pymrpt.nanogui
MRPTKMOD_ALT: mrptKeyModifier
MRPTKMOD_ALTGR: mrptKeyModifier
MRPTKMOD_CMD: mrptKeyModifier
MRPTKMOD_CONTROL: mrptKeyModifier
MRPTKMOD_META: mrptKeyModifier
MRPTKMOD_NONE: mrptKeyModifier
MRPTKMOD_SHIFT: mrptKeyModifier
MRPTKMOD_WIN: mrptKeyModifier
MRPTK_ADD: mrptKeyCode
MRPTK_ALT: mrptKeyCode
MRPTK_BACK: mrptKeyCode
MRPTK_CANCEL: mrptKeyCode
MRPTK_CAPITAL: mrptKeyCode
MRPTK_CLEAR: mrptKeyCode
MRPTK_COMMAND: mrptKeyCode
MRPTK_CONTROL: mrptKeyCode
MRPTK_DECIMAL: mrptKeyCode
MRPTK_DELETE: mrptKeyCode
MRPTK_DIVIDE: mrptKeyCode
MRPTK_DOWN: mrptKeyCode
MRPTK_END: mrptKeyCode
MRPTK_ESCAPE: mrptKeyCode
MRPTK_EXECUTE: mrptKeyCode
MRPTK_F1: mrptKeyCode
MRPTK_F10: mrptKeyCode
MRPTK_F11: mrptKeyCode
MRPTK_F12: mrptKeyCode
MRPTK_F13: mrptKeyCode
MRPTK_F14: mrptKeyCode
MRPTK_F15: mrptKeyCode
MRPTK_F16: mrptKeyCode
MRPTK_F17: mrptKeyCode
MRPTK_F18: mrptKeyCode
MRPTK_F19: mrptKeyCode
MRPTK_F2: mrptKeyCode
MRPTK_F20: mrptKeyCode
MRPTK_F21: mrptKeyCode
MRPTK_F22: mrptKeyCode
MRPTK_F23: mrptKeyCode
MRPTK_F24: mrptKeyCode
MRPTK_F3: mrptKeyCode
MRPTK_F4: mrptKeyCode
MRPTK_F5: mrptKeyCode
MRPTK_F6: mrptKeyCode
MRPTK_F7: mrptKeyCode
MRPTK_F8: mrptKeyCode
MRPTK_F9: mrptKeyCode
MRPTK_HELP: mrptKeyCode
MRPTK_HOME: mrptKeyCode
MRPTK_INSERT: mrptKeyCode
MRPTK_LBUTTON: mrptKeyCode
MRPTK_LEFT: mrptKeyCode
MRPTK_MBUTTON: mrptKeyCode
MRPTK_MENU: mrptKeyCode
MRPTK_MULTIPLY: mrptKeyCode
MRPTK_NUMLOCK: mrptKeyCode
MRPTK_NUMPAD0: mrptKeyCode
MRPTK_NUMPAD1: mrptKeyCode
MRPTK_NUMPAD2: mrptKeyCode
MRPTK_NUMPAD3: mrptKeyCode
MRPTK_NUMPAD4: mrptKeyCode
MRPTK_NUMPAD5: mrptKeyCode
MRPTK_NUMPAD6: mrptKeyCode
MRPTK_NUMPAD7: mrptKeyCode
MRPTK_NUMPAD8: mrptKeyCode
MRPTK_NUMPAD9: mrptKeyCode
MRPTK_NUMPAD_ADD: mrptKeyCode
MRPTK_NUMPAD_BEGIN: mrptKeyCode
MRPTK_NUMPAD_DECIMAL: mrptKeyCode
MRPTK_NUMPAD_DELETE: mrptKeyCode
MRPTK_NUMPAD_DIVIDE: mrptKeyCode
MRPTK_NUMPAD_DOWN: mrptKeyCode
MRPTK_NUMPAD_END: mrptKeyCode
MRPTK_NUMPAD_ENTER: mrptKeyCode
MRPTK_NUMPAD_EQUAL: mrptKeyCode
MRPTK_NUMPAD_F1: mrptKeyCode
MRPTK_NUMPAD_F2: mrptKeyCode
MRPTK_NUMPAD_F3: mrptKeyCode
MRPTK_NUMPAD_F4: mrptKeyCode
MRPTK_NUMPAD_HOME: mrptKeyCode
MRPTK_NUMPAD_INSERT: mrptKeyCode
MRPTK_NUMPAD_LEFT: mrptKeyCode
MRPTK_NUMPAD_MULTIPLY: mrptKeyCode
MRPTK_NUMPAD_PAGEDOWN: mrptKeyCode
MRPTK_NUMPAD_PAGEUP: mrptKeyCode
MRPTK_NUMPAD_RIGHT: mrptKeyCode
MRPTK_NUMPAD_SEPARATOR: mrptKeyCode
MRPTK_NUMPAD_SPACE: mrptKeyCode
MRPTK_NUMPAD_SUBTRACT: mrptKeyCode
MRPTK_NUMPAD_TAB: mrptKeyCode
MRPTK_NUMPAD_UP: mrptKeyCode
MRPTK_PAGEDOWN: mrptKeyCode
MRPTK_PAGEUP: mrptKeyCode
MRPTK_PAUSE: mrptKeyCode
MRPTK_PRINT: mrptKeyCode
MRPTK_RBUTTON: mrptKeyCode
MRPTK_RETURN: mrptKeyCode
MRPTK_RIGHT: mrptKeyCode
MRPTK_SCROLL: mrptKeyCode
MRPTK_SELECT: mrptKeyCode
MRPTK_SEPARATOR: mrptKeyCode
MRPTK_SHIFT: mrptKeyCode
MRPTK_SNAPSHOT: mrptKeyCode
MRPTK_SPACE: mrptKeyCode
MRPTK_SPECIAL1: mrptKeyCode
MRPTK_SPECIAL10: mrptKeyCode
MRPTK_SPECIAL11: mrptKeyCode
MRPTK_SPECIAL12: mrptKeyCode
MRPTK_SPECIAL13: mrptKeyCode
MRPTK_SPECIAL14: mrptKeyCode
MRPTK_SPECIAL15: mrptKeyCode
MRPTK_SPECIAL16: mrptKeyCode
MRPTK_SPECIAL17: mrptKeyCode
MRPTK_SPECIAL18: mrptKeyCode
MRPTK_SPECIAL19: mrptKeyCode
MRPTK_SPECIAL2: mrptKeyCode
MRPTK_SPECIAL20: mrptKeyCode
MRPTK_SPECIAL3: mrptKeyCode
MRPTK_SPECIAL4: mrptKeyCode
MRPTK_SPECIAL5: mrptKeyCode
MRPTK_SPECIAL6: mrptKeyCode
MRPTK_SPECIAL7: mrptKeyCode
MRPTK_SPECIAL8: mrptKeyCode
MRPTK_SPECIAL9: mrptKeyCode
MRPTK_START: mrptKeyCode
MRPTK_SUBTRACT: mrptKeyCode
MRPTK_TAB: mrptKeyCode
MRPTK_UP: mrptKeyCode
MRPTK_WINDOWS_LEFT: mrptKeyCode
MRPTK_WINDOWS_MENU: mrptKeyCode
MRPTK_WINDOWS_RIGHT: mrptKeyCode

class CBaseGUIWindow(mrpt.pymrpt.mrpt.system.CObservable):
    @overload
    def __init__(self, arg0: capsule, arg1: int, arg2: int) -> None: ...
    @overload
    def __init__(self, winobj_voidptr: capsule, CMD_CREATE_WIN: int, CMD_DESTROY_WIN: int, initial_caption: str) -> None: ...
    @overload
    def clearKeyHitFlag(self) -> None: ...
    @overload
    def clearKeyHitFlag() -> void: ...
    def getLastMousePosition(self, x: int, y: int) -> bool: ...
    @overload
    def getPushedKey(self) -> int: ...
    @overload
    def getPushedKey(self, out_pushModifier: mrptKeyModifier) -> int: ...
    @overload
    def getPushedKey(enummrpt) -> int: ...
    def getWxObject(self) -> capsule: ...
    @overload
    def isOpen(self) -> bool: ...
    @overload
    def isOpen() -> bool: ...
    @overload
    def keyHit(self) -> bool: ...
    @overload
    def keyHit() -> bool: ...
    @overload
    def notifyChildWindowDestruction(self) -> None: ...
    @overload
    def notifyChildWindowDestruction() -> void: ...
    @overload
    def notifySemThreadReady(self) -> None: ...
    @overload
    def notifySemThreadReady() -> void: ...
    def resize(self, width: int, height: int) -> None: ...
    @overload
    def setCursorCross(self, cursorIsCross: bool) -> None: ...
    @overload
    def setCursorCross(bool) -> void: ...
    def setPos(self, x: int, y: int) -> None: ...
    @overload
    def setWindowTitle(self, str: str) -> None: ...
    @overload
    def setWindowTitle(conststd) -> void: ...
    @overload
    def waitForKey(self) -> int: ...
    @overload
    def waitForKey(self, ignoreControlKeys: bool) -> int: ...
    @overload
    def waitForKey(self, ignoreControlKeys: bool, out_pushModifier: mrptKeyModifier) -> int: ...
    @overload
    def waitForKey(bool, enummrpt) -> int: ...

class CDisplayWindow(CBaseGUIWindow):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: str) -> None: ...
    @overload
    def __init__(self, arg0: str, arg1: int) -> None: ...
    @overload
    def __init__(self, windowCaption: str, initWidth: int, initHeight: int) -> None: ...
    def Create(self, *args, **kwargs) -> Any: ...
    @overload
    def enableCursorCoordinatesVisualization(self, enable: bool) -> None: ...
    @overload
    def enableCursorCoordinatesVisualization(bool) -> void: ...
    def getLastMousePosition(self, x: int, y: int) -> bool: ...
    @overload
    def plot(self, x, y) -> None: ...
    @overload
    def plot(self, y) -> None: ...
    @overload
    def plot(constclassmrpt) -> void: ...
    def resize(self, width: int, height: int) -> None: ...
    @overload
    def setCursorCross(self, cursorIsCross: bool) -> None: ...
    @overload
    def setCursorCross(bool) -> void: ...
    def setPos(self, x: int, y: int) -> None: ...
    @overload
    def setWindowTitle(self, str: str) -> None: ...
    @overload
    def setWindowTitle(conststd) -> void: ...
    @overload
    def showImage(self, img: mrpt.pymrpt.mrpt.img.CImage) -> None: ...
    @overload
    def showImage(constclassmrpt) -> void: ...
    @overload
    def showImageAndPoints(self, img: mrpt.pymrpt.mrpt.img.CImage, x, y) -> None: ...
    @overload
    def showImageAndPoints(self, img: mrpt.pymrpt.mrpt.img.CImage, x, y, color: mrpt.pymrpt.mrpt.img.TColor) -> None: ...
    @overload
    def showImageAndPoints(self, img: mrpt.pymrpt.mrpt.img.CImage, x, y, color: mrpt.pymrpt.mrpt.img.TColor, showNumbers: bool) -> None: ...

class CDisplayWindow3D(CBaseGUIWindow):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: str) -> None: ...
    @overload
    def __init__(self, arg0: str, arg1: int) -> None: ...
    @overload
    def __init__(self, windowCaption: str, initialWindowWidth: int, initialWindowHeight: int) -> None: ...
    def Create(self, *args, **kwargs) -> Any: ...
    @overload
    def addTextMessage(self, x_frac: float, y_frac: float, text: str) -> None: ...
    @overload
    def addTextMessage(self, x_frac: float, y_frac: float, text: str, unique_index: int) -> None: ...
    @overload
    def addTextMessage(self, x_frac: float, y_frac: float, text: str, unique_index: int, fontParams: mrpt.pymrpt.mrpt.opengl.TFontParams) -> None: ...
    @overload
    def addTextMessage() -> Any: ...
    @overload
    def addTextMessage() -> Any: ...
    @overload
    def captureImagesStart(self) -> None: ...
    @overload
    def captureImagesStart() -> void: ...
    @overload
    def captureImagesStop(self) -> None: ...
    @overload
    def captureImagesStop() -> void: ...
    @overload
    def clearTextMessages(self) -> None: ...
    @overload
    def clearTextMessages() -> Any: ...
    @overload
    def clearTextMessages() -> void: ...
    @overload
    def forceRepaint(self) -> None: ...
    @overload
    def forceRepaint() -> void: ...
    def get3DSceneAndLock(self) -> mrpt.pymrpt.mrpt.opengl.Scene: ...
    @overload
    def getCameraAzimuthDeg(self) -> float: ...
    @overload
    def getCameraAzimuthDeg() -> float: ...
    @overload
    def getCameraElevationDeg(self) -> float: ...
    @overload
    def getCameraElevationDeg() -> float: ...
    def getCameraPointingToPoint(self, x: float, y: float, z: float) -> None: ...
    @overload
    def getCameraZoom(self) -> float: ...
    @overload
    def getCameraZoom() -> float: ...
    def getDefaultViewport(self) -> mrpt.pymrpt.mrpt.opengl.Viewport: ...
    @overload
    def getFOV(self) -> float: ...
    @overload
    def getFOV() -> float: ...
    def getLastMousePosition(self, x: int, y: int) -> bool: ...
    @overload
    def getLastMousePositionRay(self, ray) -> bool: ...
    @overload
    def getLastMousePositionRay(structmrpt) -> bool: ...
    def getLastWindowImage(self, *args, **kwargs) -> Any: ...
    def getLastWindowImagePtr(self, *args, **kwargs) -> Any: ...
    @overload
    def getRenderingFPS(self) -> float: ...
    @overload
    def getRenderingFPS() -> double: ...
    def grabImageGetNextFile(self) -> str: ...
    def grabImagesStart(self, *args, **kwargs) -> Any: ...
    @overload
    def grabImagesStop(self) -> None: ...
    @overload
    def grabImagesStop() -> void: ...
    @overload
    def isCameraProjective(self) -> bool: ...
    @overload
    def isCameraProjective() -> bool: ...
    @overload
    def isCapturingImgs(self) -> bool: ...
    @overload
    def isCapturingImgs() -> bool: ...
    @overload
    def is_GL_context_created(self) -> bool: ...
    @overload
    def is_GL_context_created() -> bool: ...
    @overload
    def repaint(self) -> None: ...
    @overload
    def repaint() -> void: ...
    def resize(self, width: int, height: int) -> None: ...
    @overload
    def sendFunctionToRunOnGUIThread(self, f) -> None: ...
    @overload
    def sendFunctionToRunOnGUIThread(constclassstd) -> void: ...
    @overload
    def setCameraAzimuthDeg(self, deg: float) -> None: ...
    @overload
    def setCameraAzimuthDeg(float) -> void: ...
    @overload
    def setCameraElevationDeg(self, deg: float) -> None: ...
    @overload
    def setCameraElevationDeg(float) -> void: ...
    def setCameraPointingToPoint(self, x: float, y: float, z: float) -> None: ...
    @overload
    def setCameraProjective(self, isProjective: bool) -> None: ...
    @overload
    def setCameraProjective(bool) -> void: ...
    @overload
    def setCameraZoom(self, zoom: float) -> None: ...
    @overload
    def setCameraZoom(float) -> void: ...
    @overload
    def setCursorCross(self, cursorIsCross: bool) -> None: ...
    @overload
    def setCursorCross(bool) -> void: ...
    @overload
    def setFOV(self, v: float) -> None: ...
    @overload
    def setFOV(float) -> void: ...
    @overload
    def setImageView(self, img: mrpt.pymrpt.mrpt.img.CImage) -> None: ...
    @overload
    def setImageView(constclassmrpt) -> void: ...
    def setMaxRange(self, new_max: float) -> None: ...
    @overload
    def setMinRange(self, new_min: float) -> None: ...
    @overload
    def setMinRange(float) -> void: ...
    def setPos(self, x: int, y: int) -> None: ...
    @overload
    def setWindowTitle(self, str: str) -> None: ...
    @overload
    def setWindowTitle(conststd) -> void: ...
    @overload
    def unlockAccess3DScene(self) -> None: ...
    @overload
    def unlockAccess3DScene() -> void: ...
    @overload
    def updateTextMessage(self, unique_index: int, text: str) -> bool: ...
    @overload
    def updateTextMessage() -> Any: ...
    @overload
    def updateTextMessage(constunsignedlong, conststd) -> bool: ...
    @overload
    def updateWindow(self) -> None: ...
    @overload
    def updateWindow() -> void: ...
    @overload
    def useCameraFromScene(self) -> None: ...
    @overload
    def useCameraFromScene(self, useIt: bool) -> None: ...
    @overload
    def useCameraFromScene(bool) -> void: ...
    @overload
    def wait_for_GL_context(self) -> bool: ...
    @overload
    def wait_for_GL_context(self, timeout_seconds: float) -> bool: ...
    @overload
    def wait_for_GL_context(constdouble) -> bool: ...

class CDisplayWindow3DLocker:
    @overload
    def __init__(self, win: CDisplayWindow3D, out_scene_ptr: mrpt.pymrpt.mrpt.opengl.Scene) -> None: ...
    @overload
    def __init__(self, win: CDisplayWindow3D) -> None: ...

class CDisplayWindowGUI(mrpt.pymrpt.nanogui.Screen):
    background_scene: mrpt.pymrpt.mrpt.opengl.Scene
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: str) -> None: ...
    @overload
    def __init__(self, arg0: str, arg1: int) -> None: ...
    @overload
    def __init__(self, arg0: str, arg1: int, arg2: int) -> None: ...
    @overload
    def __init__(self, caption: str, width: int, height: int, p: CDisplayWindowGUI_Params) -> None: ...
    @overload
    def addDropFilesCallback(self, callback) -> None: ...
    @overload
    def addDropFilesCallback(constclassstd) -> void: ...
    @overload
    def addKeyboardCallback(self, callback) -> None: ...
    @overload
    def addKeyboardCallback(constclassstd) -> void: ...
    @overload
    def addLoopCallback(self, callback) -> None: ...
    @overload
    def addLoopCallback(constclassstd) -> void: ...
    def camera(self) -> CGlCanvasBase: ...
    def createManagedSubWindow(self, title: str) -> mrpt.pymrpt.nanogui.Window: ...
    @overload
    def drawContents(self) -> None: ...
    @overload
    def drawContents() -> void: ...
    def dropFilesCallback(self, *args, **kwargs) -> Any: ...
    def getSubWindowsUI(self) -> mrpt.pymrpt.nanogui.Window: ...
    def getSubwindow(self, index: int) -> mrpt.pymrpt.nanogui.Window: ...
    @overload
    def getSubwindowCount(self) -> int: ...
    @overload
    def getSubwindowCount() -> size_t: ...
    def keyboardCallback(self, *args, **kwargs) -> Any: ...
    def loopCallback(self, *args, **kwargs) -> Any: ...
    def nanogui_screen(self) -> mrpt.pymrpt.nanogui.Screen: ...
    @overload
    def onIdleLoopTasks(self) -> None: ...
    @overload
    def onIdleLoopTasks() -> void: ...
    def resize(self, width: int, height: int) -> None: ...
    @overload
    def setDropFilesCallback(self, callback) -> None: ...
    @overload
    def setDropFilesCallback(constclassstd) -> void: ...
    @overload
    def setIcon(self, img: mrpt.pymrpt.mrpt.img.CImage) -> None: ...
    @overload
    def setIcon(constclassmrpt) -> void: ...
    def setIconFromData(self, imgData: str, width: int, height: int, transparent: int) -> None: ...
    @overload
    def setKeyboardCallback(self, callback) -> None: ...
    @overload
    def setKeyboardCallback(constclassstd) -> void: ...
    @overload
    def setLoopCallback(self, callback) -> None: ...
    @overload
    def setLoopCallback(constclassstd) -> void: ...
    def setPos(self, x: int, y: int) -> None: ...
    @overload
    def setWindowTitle(self, str: str) -> None: ...
    @overload
    def setWindowTitle(conststd) -> void: ...
    @overload
    def subwindowMinimize(self, index: int) -> None: ...
    @overload
    def subwindowMinimize(size_t) -> void: ...
    @overload
    def subwindowRestore(self, index: int) -> None: ...
    @overload
    def subwindowRestore(size_t) -> void: ...
    @overload
    def subwindowSetFocused(self, index: int) -> None: ...
    @overload
    def subwindowSetFocused(size_t) -> void: ...

class CDisplayWindowGUI_Params:
    alphaBits: int
    colorBits: int
    depthBits: int
    fullscreen: bool
    glMajor: int
    glMinor: int
    gles_context: bool
    maximized: bool
    nSamples: int
    resizable: bool
    stencilBits: int
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CDisplayWindowGUI_Params) -> None: ...

class CDisplayWindowPlots(CBaseGUIWindow):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: str) -> None: ...
    @overload
    def __init__(self, arg0: str, arg1: int) -> None: ...
    @overload
    def __init__(self, windowCaption: str, initialWidth: int, initialHeight: int) -> None: ...
    def Create(self, *args, **kwargs) -> Any: ...
    @overload
    def addPopupMenuEntry(self, label: str, menuID: int) -> None: ...
    @overload
    def addPopupMenuEntry(conststd, int) -> void: ...
    @overload
    def axis(self, x_min: float, x_max: float, y_min: float, y_max: float) -> None: ...
    @overload
    def axis(self, x_min: float, x_max: float, y_min: float, y_max: float, aspectRatioFix: bool) -> None: ...
    @overload
    def axis_equal(self) -> None: ...
    @overload
    def axis_equal(self, enable: bool) -> None: ...
    @overload
    def axis_equal(bool) -> void: ...
    @overload
    def axis_fit(self) -> None: ...
    @overload
    def axis_fit(self, aspectRatioFix: bool) -> None: ...
    @overload
    def axis_fit(bool) -> void: ...
    @overload
    def clear(self) -> None: ...
    @overload
    def clear() -> void: ...
    @overload
    def clf(self) -> None: ...
    @overload
    def clf() -> void: ...
    @overload
    def enableMousePanZoom(self, enabled: bool) -> None: ...
    @overload
    def enableMousePanZoom(bool) -> void: ...
    def getLastMousePosition(self, x: int, y: int) -> bool: ...
    @overload
    def hold_off(self) -> None: ...
    @overload
    def hold_off() -> void: ...
    @overload
    def hold_on(self) -> None: ...
    @overload
    def hold_on() -> void: ...
    def image(self, *args, **kwargs) -> Any: ...
    def resize(self, width: int, height: int) -> None: ...
    @overload
    def setCursorCross(self, cursorIsCross: bool) -> None: ...
    @overload
    def setCursorCross(bool) -> void: ...
    def setPos(self, x: int, y: int) -> None: ...
    @overload
    def setWindowTitle(self, str: str) -> None: ...
    @overload
    def setWindowTitle(conststd) -> void: ...

class CGlCanvasBase:
    class CamaraParams:
        cameraAzimuthDeg: float
        cameraElevationDeg: float
        cameraFOV: float
        cameraIsProjective: bool
        cameraPointingX: float
        cameraPointingY: float
        cameraPointingZ: float
        cameraRollDeg: float
        cameraZoomDistance: float
        @overload
        def __init__(self) -> None: ...
        @overload
        def __init__(self, arg0: CGlCanvasBase.CamaraParams) -> None: ...
        def FromCamera(self, *args, **kwargs) -> Any: ...
        def assign(self) -> CGlCanvasBase.CamaraParams: ...
        @overload
        def setElevationDeg(self, deg: float) -> None: ...
        @overload
        def setElevationDeg(float) -> void: ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CGlCanvasBase) -> None: ...
    def OnUserManuallyMovesCamera(self, new_cameraPointingX: float, new_cameraPointingY: float, new_cameraPointingZ: float, new_cameraZoomDistance: float, new_cameraElevationDeg: float, new_cameraAzimuthDeg: float) -> None: ...
    def assign(self) -> CGlCanvasBase: ...
    @overload
    def cameraFOV(self) -> float: ...
    @overload
    def cameraFOV() -> float: ...
    def cameraParams(self, *args, **kwargs) -> Any: ...
    @overload
    def getAzimuthDegrees(self) -> float: ...
    @overload
    def getAzimuthDegrees() -> float: ...
    @overload
    def getCameraPointingX(self) -> float: ...
    @overload
    def getCameraPointingX() -> float: ...
    @overload
    def getCameraPointingY(self) -> float: ...
    @overload
    def getCameraPointingY() -> float: ...
    @overload
    def getCameraPointingZ(self) -> float: ...
    @overload
    def getCameraPointingZ() -> float: ...
    @overload
    def getElevationDegrees(self) -> float: ...
    @overload
    def getElevationDegrees() -> float: ...
    def getLastMousePosition(self, x: int, y: int) -> None: ...
    def getOpenGLSceneRef(self, *args, **kwargs) -> Any: ...
    def getRefCameraParams(self, *args, **kwargs) -> Any: ...
    @overload
    def getRollDegrees(self) -> float: ...
    @overload
    def getRollDegrees() -> float: ...
    @overload
    def getUseCameraFromScene(self) -> bool: ...
    @overload
    def getUseCameraFromScene() -> bool: ...
    @overload
    def getZoomDistance(self) -> float: ...
    @overload
    def getZoomDistance() -> float: ...
    @overload
    def isCameraProjective(self) -> bool: ...
    @overload
    def isCameraProjective() -> bool: ...
    def resizeViewport(self, w: int, h: int) -> None: ...
    @overload
    def setAzimuthDegrees(self, ang: float) -> None: ...
    @overload
    def setAzimuthDegrees(float) -> void: ...
    @overload
    def setCameraFOV(self, FOV: float) -> None: ...
    @overload
    def setCameraFOV(float) -> void: ...
    @overload
    def setCameraParams(self, params) -> None: ...
    @overload
    def setCameraParams(conststructmrpt) -> void: ...
    def setCameraPointing(self, pointX: float, pointY: float, pointZ: float) -> None: ...
    @overload
    def setCameraProjective(self, is: bool) -> None: ...
    @overload
    def setCameraProjective(bool) -> void: ...
    @overload
    def setElevationDegrees(self, ang: float) -> None: ...
    @overload
    def setElevationDegrees(float) -> void: ...
    @overload
    def setMaximumZoom(self, zoom: float) -> None: ...
    @overload
    def setMaximumZoom(float) -> void: ...
    @overload
    def setMinimumZoom(self, zoom: float) -> None: ...
    @overload
    def setMinimumZoom(float) -> void: ...
    @overload
    def setMouseClicked(self, is: bool) -> None: ...
    @overload
    def setMouseClicked(bool) -> void: ...
    def setMousePos(self, x: int, y: int) -> None: ...
    @overload
    def setOpenGLSceneRef(self, scene: mrpt.pymrpt.mrpt.opengl.Scene) -> None: ...
    @overload
    def setOpenGLSceneRef(classstd) -> void: ...
    @overload
    def setUseCameraFromScene(self, is: bool) -> None: ...
    @overload
    def setUseCameraFromScene(bool) -> void: ...
    @overload
    def setZoomDistance(self, zoom: float) -> None: ...
    @overload
    def setZoomDistance(float) -> void: ...
    def updateCameraParams(self, cam: mrpt.pymrpt.mrpt.opengl.CCamera) -> mrpt.pymrpt.mrpt.opengl.CCamera: ...
    def updateLastPos(self, x: int, y: int) -> None: ...
    def updateOrbitCamera(self, params, x: int, y: int) -> None: ...
    def updatePan(self, params, x: int, y: int) -> None: ...
    @overload
    def updateRoll(self, params, x: int, y: int) -> None: ...
    @overload
    def updateRoll(self, params, x: int, y: int, scale: float) -> None: ...
    def updateRotate(self, params, x: int, y: int) -> None: ...
    @overload
    def updateZoom(self, params, x: int, y: int) -> None: ...
    @overload
    def updateZoom(self, params, delta: float) -> None: ...
    @overload
    def updateZoom(structmrpt, float) -> void: ...

class CGlCanvasBaseHeadless(CGlCanvasBase):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CGlCanvasBaseHeadless) -> None: ...
    @overload
    def __init__(self, arg0: CGlCanvasBaseHeadless) -> None: ...
    def assign(self) -> CGlCanvasBaseHeadless: ...

class MRPT2NanoguiGLCanvas(mrpt.pymrpt.nanogui.GLCanvas):
    scene: mrpt.pymrpt.mrpt.opengl.Scene
    def __init__(self, *args, **kwargs) -> None: ...
    def camera(self) -> CGlCanvasBase: ...
    @overload
    def drawGL(self) -> None: ...
    @overload
    def drawGL() -> void: ...

class mrptEvent3DWindowGrabImageFile(mrpt.pymrpt.mrpt.system.mrptEvent):
    def __init__(self, obj: CDisplayWindow3D, _img_file: str) -> None: ...

class mrptEventMouseDown(mrpt.pymrpt.mrpt.system.mrptEvent):
    coords: mrpt.pymrpt.mrpt.img.TPixelCoord
    leftButton: bool
    rightButton: bool
    def __init__(self, obj: CBaseGUIWindow, _coords: mrpt.pymrpt.mrpt.img.TPixelCoord, _leftButton: bool, _rightButton: bool) -> None: ...
    def assign(self) -> mrptEventMouseDown: ...

class mrptEventMouseMove(mrpt.pymrpt.mrpt.system.mrptEvent):
    coords: mrpt.pymrpt.mrpt.img.TPixelCoord
    leftButton: bool
    rightButton: bool
    def __init__(self, obj: CBaseGUIWindow, _coords: mrpt.pymrpt.mrpt.img.TPixelCoord, _leftButton: bool, _rightButton: bool) -> None: ...
    def assign(self) -> mrptEventMouseMove: ...

class mrptEventWindowChar(mrpt.pymrpt.mrpt.system.mrptEvent):
    char_code: int
    key_modifiers: mrptKeyModifier
    def __init__(self, obj: CBaseGUIWindow, _char_code: int, _key_mod: mrptKeyModifier) -> None: ...
    def assign(self) -> mrptEventWindowChar: ...

class mrptEventWindowClosed(mrpt.pymrpt.mrpt.system.mrptEvent):
    allow_close: bool
    @overload
    def __init__(self, arg0: CBaseGUIWindow) -> None: ...
    @overload
    def __init__(self, obj: CBaseGUIWindow, _allow_close: bool) -> None: ...
    def assign(self) -> mrptEventWindowClosed: ...

class mrptEventWindowResize(mrpt.pymrpt.mrpt.system.mrptEvent):
    new_height: int
    new_width: int
    def __init__(self, obj: CBaseGUIWindow, _new_width: int, _new_height: int) -> None: ...
    def assign(self) -> mrptEventWindowResize: ...

class mrptKeyCode:
    __doc__: ClassVar[str] = ...  # read-only
    __members__: ClassVar[dict] = ...  # read-only
    MRPTK_ADD: ClassVar[mrptKeyCode] = ...
    MRPTK_ALT: ClassVar[mrptKeyCode] = ...
    MRPTK_BACK: ClassVar[mrptKeyCode] = ...
    MRPTK_CANCEL: ClassVar[mrptKeyCode] = ...
    MRPTK_CAPITAL: ClassVar[mrptKeyCode] = ...
    MRPTK_CLEAR: ClassVar[mrptKeyCode] = ...
    MRPTK_COMMAND: ClassVar[mrptKeyCode] = ...
    MRPTK_CONTROL: ClassVar[mrptKeyCode] = ...
    MRPTK_DECIMAL: ClassVar[mrptKeyCode] = ...
    MRPTK_DELETE: ClassVar[mrptKeyCode] = ...
    MRPTK_DIVIDE: ClassVar[mrptKeyCode] = ...
    MRPTK_DOWN: ClassVar[mrptKeyCode] = ...
    MRPTK_END: ClassVar[mrptKeyCode] = ...
    MRPTK_ESCAPE: ClassVar[mrptKeyCode] = ...
    MRPTK_EXECUTE: ClassVar[mrptKeyCode] = ...
    MRPTK_F1: ClassVar[mrptKeyCode] = ...
    MRPTK_F10: ClassVar[mrptKeyCode] = ...
    MRPTK_F11: ClassVar[mrptKeyCode] = ...
    MRPTK_F12: ClassVar[mrptKeyCode] = ...
    MRPTK_F13: ClassVar[mrptKeyCode] = ...
    MRPTK_F14: ClassVar[mrptKeyCode] = ...
    MRPTK_F15: ClassVar[mrptKeyCode] = ...
    MRPTK_F16: ClassVar[mrptKeyCode] = ...
    MRPTK_F17: ClassVar[mrptKeyCode] = ...
    MRPTK_F18: ClassVar[mrptKeyCode] = ...
    MRPTK_F19: ClassVar[mrptKeyCode] = ...
    MRPTK_F2: ClassVar[mrptKeyCode] = ...
    MRPTK_F20: ClassVar[mrptKeyCode] = ...
    MRPTK_F21: ClassVar[mrptKeyCode] = ...
    MRPTK_F22: ClassVar[mrptKeyCode] = ...
    MRPTK_F23: ClassVar[mrptKeyCode] = ...
    MRPTK_F24: ClassVar[mrptKeyCode] = ...
    MRPTK_F3: ClassVar[mrptKeyCode] = ...
    MRPTK_F4: ClassVar[mrptKeyCode] = ...
    MRPTK_F5: ClassVar[mrptKeyCode] = ...
    MRPTK_F6: ClassVar[mrptKeyCode] = ...
    MRPTK_F7: ClassVar[mrptKeyCode] = ...
    MRPTK_F8: ClassVar[mrptKeyCode] = ...
    MRPTK_F9: ClassVar[mrptKeyCode] = ...
    MRPTK_HELP: ClassVar[mrptKeyCode] = ...
    MRPTK_HOME: ClassVar[mrptKeyCode] = ...
    MRPTK_INSERT: ClassVar[mrptKeyCode] = ...
    MRPTK_LBUTTON: ClassVar[mrptKeyCode] = ...
    MRPTK_LEFT: ClassVar[mrptKeyCode] = ...
    MRPTK_MBUTTON: ClassVar[mrptKeyCode] = ...
    MRPTK_MENU: ClassVar[mrptKeyCode] = ...
    MRPTK_MULTIPLY: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMLOCK: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD0: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD1: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD2: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD3: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD4: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD5: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD6: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD7: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD8: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD9: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_ADD: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_BEGIN: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_DECIMAL: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_DELETE: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_DIVIDE: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_DOWN: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_END: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_ENTER: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_EQUAL: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_F1: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_F2: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_F3: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_F4: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_HOME: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_INSERT: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_LEFT: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_MULTIPLY: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_PAGEDOWN: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_PAGEUP: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_RIGHT: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_SEPARATOR: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_SPACE: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_SUBTRACT: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_TAB: ClassVar[mrptKeyCode] = ...
    MRPTK_NUMPAD_UP: ClassVar[mrptKeyCode] = ...
    MRPTK_PAGEDOWN: ClassVar[mrptKeyCode] = ...
    MRPTK_PAGEUP: ClassVar[mrptKeyCode] = ...
    MRPTK_PAUSE: ClassVar[mrptKeyCode] = ...
    MRPTK_PRINT: ClassVar[mrptKeyCode] = ...
    MRPTK_RBUTTON: ClassVar[mrptKeyCode] = ...
    MRPTK_RETURN: ClassVar[mrptKeyCode] = ...
    MRPTK_RIGHT: ClassVar[mrptKeyCode] = ...
    MRPTK_SCROLL: ClassVar[mrptKeyCode] = ...
    MRPTK_SELECT: ClassVar[mrptKeyCode] = ...
    MRPTK_SEPARATOR: ClassVar[mrptKeyCode] = ...
    MRPTK_SHIFT: ClassVar[mrptKeyCode] = ...
    MRPTK_SNAPSHOT: ClassVar[mrptKeyCode] = ...
    MRPTK_SPACE: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL1: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL10: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL11: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL12: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL13: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL14: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL15: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL16: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL17: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL18: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL19: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL2: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL20: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL3: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL4: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL5: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL6: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL7: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL8: ClassVar[mrptKeyCode] = ...
    MRPTK_SPECIAL9: ClassVar[mrptKeyCode] = ...
    MRPTK_START: ClassVar[mrptKeyCode] = ...
    MRPTK_SUBTRACT: ClassVar[mrptKeyCode] = ...
    MRPTK_TAB: ClassVar[mrptKeyCode] = ...
    MRPTK_UP: ClassVar[mrptKeyCode] = ...
    MRPTK_WINDOWS_LEFT: ClassVar[mrptKeyCode] = ...
    MRPTK_WINDOWS_MENU: ClassVar[mrptKeyCode] = ...
    MRPTK_WINDOWS_RIGHT: ClassVar[mrptKeyCode] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __and__(self, other: object) -> object: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> object: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __or__(self, other: object) -> object: ...
    def __rand__(self, other: object) -> object: ...
    def __ror__(self, other: object) -> object: ...
    def __rxor__(self, other: object) -> object: ...
    def __setstate__(self, state: int) -> None: ...
    def __xor__(self, other: object) -> object: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class mrptKeyModifier:
    __doc__: ClassVar[str] = ...  # read-only
    __members__: ClassVar[dict] = ...  # read-only
    MRPTKMOD_ALT: ClassVar[mrptKeyModifier] = ...
    MRPTKMOD_ALTGR: ClassVar[mrptKeyModifier] = ...
    MRPTKMOD_CMD: ClassVar[mrptKeyModifier] = ...
    MRPTKMOD_CONTROL: ClassVar[mrptKeyModifier] = ...
    MRPTKMOD_META: ClassVar[mrptKeyModifier] = ...
    MRPTKMOD_NONE: ClassVar[mrptKeyModifier] = ...
    MRPTKMOD_SHIFT: ClassVar[mrptKeyModifier] = ...
    MRPTKMOD_WIN: ClassVar[mrptKeyModifier] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __and__(self, other: object) -> object: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> object: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __or__(self, other: object) -> object: ...
    def __rand__(self, other: object) -> object: ...
    def __ror__(self, other: object) -> object: ...
    def __rxor__(self, other: object) -> object: ...
    def __setstate__(self, state: int) -> None: ...
    def __xor__(self, other: object) -> object: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...



from typing import Dict

from typing import overload
import mrpt.pymrpt.mrpt.config
import mrpt.pymrpt.mrpt.math
import mrpt.pymrpt.mrpt.poses
import mrpt.pymrpt.mrpt.slam
import mrpt.pymrpt.std.chrono

class BaseAppDataSource:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: BaseAppDataSource) -> None: ...
    def assign(self) -> BaseAppDataSource: ...

class BaseAppInitializableCLI:
    def __init__(self, *args, **kwargs) -> None: ...
    def assign(self) -> BaseAppInitializableCLI: ...

class DataSourceRawlog(BaseAppDataSource):
    def __init__(self) -> None: ...

class ICP_SLAM_App_Base(BaseAppInitializableCLI, BaseAppDataSource):
    out_estimated_path: Dict[mrpt.pymrpt.std.chrono.time_point_mrpt_Clock_std_chrono_duration_long_std_ratio_1_10000000_t,mrpt.pymrpt.mrpt.math.TPose3D]
    params: mrpt.pymrpt.mrpt.config.CConfigFileMemory
    quits_with_esc_key: bool
    def __init__(self, *args, **kwargs) -> None: ...
    def assign(self) -> ICP_SLAM_App_Base: ...
    @overload
    def run(self) -> None: ...
    @overload
    def run() -> void: ...

class ICP_SLAM_App_Live(ICP_SLAM_App_Base):
    def __init__(self) -> None: ...

class ICP_SLAM_App_Rawlog(ICP_SLAM_App_Base, DataSourceRawlog):
    def __init__(self) -> None: ...

class KFSLAMApp:
    loc_error_wrt_gt: float
    params: mrpt.pymrpt.mrpt.config.CConfigFileMemory
    rawlogFileName: str
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: KFSLAMApp) -> None: ...
    def assign(self) -> KFSLAMApp: ...
    @overload
    def run(self) -> None: ...
    @overload
    def run() -> void: ...

class MonteCarloLocalization_Base(BaseAppInitializableCLI, BaseAppDataSource):
    allow_quit_on_esc_key: bool
    fill_out_estimated_path: bool
    out_estimated_path: mrpt.pymrpt.mrpt.poses.CPose3DInterpolator
    params: mrpt.pymrpt.mrpt.config.CConfigFileMemory
    def __init__(self, *args, **kwargs) -> None: ...
    def assign(self) -> MonteCarloLocalization_Base: ...
    @overload
    def run(self) -> None: ...
    @overload
    def run() -> void: ...

class MonteCarloLocalization_Rawlog(MonteCarloLocalization_Base, DataSourceRawlog):
    def __init__(self) -> None: ...

class RBPF_SLAM_App_Base(BaseAppInitializableCLI, BaseAppDataSource):
    mapBuilder: mrpt.pymrpt.mrpt.slam.CMetricMapBuilderRBPF
    out_estimated_path: Dict[mrpt.pymrpt.std.chrono.time_point_mrpt_Clock_std_chrono_duration_long_std_ratio_1_10000000_t,mrpt.pymrpt.mrpt.math.TPose3D]
    params: mrpt.pymrpt.mrpt.config.CConfigFileMemory
    quits_with_esc_key: bool
    def __init__(self, *args, **kwargs) -> None: ...
    def assign(self) -> RBPF_SLAM_App_Base: ...
    @overload
    def run(self) -> None: ...
    @overload
    def run() -> void: ...

class RBPF_SLAM_App_Rawlog(RBPF_SLAM_App_Base, DataSourceRawlog):
    def __init__(self) -> None: ...
    def init(self, iniConfigFile: str, rawlogFile: str) -> None: ...

class RawlogEditApp:
    def __init__(self) -> None: ...

class RawlogGrabberApp:
    params: mrpt.pymrpt.mrpt.config.CConfigFileMemory
    rawlog_filename: str
    rawlog_saved_objects: int
    run_for_seconds: float
    show_sensor_thread_exceptions: bool
    def __init__(self) -> None: ...
    @overload
    def SensorThread(self, sensor_label: str) -> None: ...
    @overload
    def SensorThread(std) -> void: ...
    @overload
    def isRunning(self) -> bool: ...
    @overload
    def isRunning() -> bool: ...
    @overload
    def run(self) -> None: ...
    @overload
    def run() -> void: ...



from typing import Any, List

from typing import overload
import mrpt.pymrpt.mrpt.io
import mrpt.pymrpt.mrpt.rtti

class CArchive:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CArchive) -> None: ...
    @overload
    def ReadAs(self) -> int: ...
    @overload
    def ReadAs() -> unsignedint: ...
    def ReadBuffer(self, Buffer: capsule, Count: int) -> int: ...
    def ReadObject(self, *args, **kwargs) -> Any: ...
    def WriteAs(self, value: int) -> CArchive: ...
    def WriteBuffer(self, Buffer: capsule, Count: int) -> None: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    def assign(self) -> CArchive: ...
    def getArchiveDescription(self) -> str: ...
    @overload
    def __lshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __lshift__(self, pObj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, pObj: CSerializable) -> CArchive: ...

class CArchiveStreamBase_mrpt_io_CFileGZInputStream_t(CArchive):
    @overload
    def __init__(self, s: mrpt.pymrpt.mrpt.io.CFileGZInputStream) -> None: ...
    @overload
    def __init__(self, arg0: CArchiveStreamBase_mrpt_io_CFileGZInputStream_t) -> None: ...
    @overload
    def __init__(self, arg0: CArchiveStreamBase_mrpt_io_CFileGZInputStream_t) -> None: ...
    @overload
    def ReadAs(self) -> int: ...
    @overload
    def ReadAs() -> unsignedint: ...
    def ReadBuffer(self, Buffer: capsule, Count: int) -> int: ...
    def ReadObject(self, *args, **kwargs) -> Any: ...
    def WriteAs(self, value: int) -> CArchive: ...
    def WriteBuffer(self, Buffer: capsule, Count: int) -> None: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    def assign(self) -> CArchive: ...
    @overload
    def getArchiveDescription(self) -> str: ...
    @overload
    def getArchiveDescription(self) -> str: ...
    @overload
    def __lshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __lshift__(self, pObj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, pObj: CSerializable) -> CArchive: ...

class CArchiveStreamBase_mrpt_io_CFileGZOutputStream_t(CArchive):
    @overload
    def __init__(self, s: mrpt.pymrpt.mrpt.io.CFileGZOutputStream) -> None: ...
    @overload
    def __init__(self, arg0: CArchiveStreamBase_mrpt_io_CFileGZOutputStream_t) -> None: ...
    @overload
    def __init__(self, arg0: CArchiveStreamBase_mrpt_io_CFileGZOutputStream_t) -> None: ...
    @overload
    def ReadAs(self) -> int: ...
    @overload
    def ReadAs() -> unsignedint: ...
    def ReadBuffer(self, Buffer: capsule, Count: int) -> int: ...
    def ReadObject(self, *args, **kwargs) -> Any: ...
    def WriteAs(self, value: int) -> CArchive: ...
    def WriteBuffer(self, Buffer: capsule, Count: int) -> None: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    def assign(self) -> CArchive: ...
    @overload
    def getArchiveDescription(self) -> str: ...
    @overload
    def getArchiveDescription(self) -> str: ...
    @overload
    def __lshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __lshift__(self, pObj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, pObj: CSerializable) -> CArchive: ...

class CArchiveStreamBase_mrpt_io_CFileInputStream_t(CArchive):
    @overload
    def __init__(self, s: mrpt.pymrpt.mrpt.io.CFileInputStream) -> None: ...
    @overload
    def __init__(self, arg0: CArchiveStreamBase_mrpt_io_CFileInputStream_t) -> None: ...
    @overload
    def __init__(self, arg0: CArchiveStreamBase_mrpt_io_CFileInputStream_t) -> None: ...
    @overload
    def ReadAs(self) -> int: ...
    @overload
    def ReadAs() -> unsignedint: ...
    def ReadBuffer(self, Buffer: capsule, Count: int) -> int: ...
    def ReadObject(self, *args, **kwargs) -> Any: ...
    def WriteAs(self, value: int) -> CArchive: ...
    def WriteBuffer(self, Buffer: capsule, Count: int) -> None: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    def assign(self) -> CArchive: ...
    @overload
    def getArchiveDescription(self) -> str: ...
    @overload
    def getArchiveDescription(self) -> str: ...
    @overload
    def __lshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __lshift__(self, pObj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, pObj: CSerializable) -> CArchive: ...

class CArchiveStreamBase_mrpt_io_CFileOutputStream_t(CArchive):
    @overload
    def __init__(self, s: mrpt.pymrpt.mrpt.io.CFileOutputStream) -> None: ...
    @overload
    def __init__(self, arg0: CArchiveStreamBase_mrpt_io_CFileOutputStream_t) -> None: ...
    @overload
    def __init__(self, arg0: CArchiveStreamBase_mrpt_io_CFileOutputStream_t) -> None: ...
    @overload
    def ReadAs(self) -> int: ...
    @overload
    def ReadAs() -> unsignedint: ...
    def ReadBuffer(self, Buffer: capsule, Count: int) -> int: ...
    def ReadObject(self, *args, **kwargs) -> Any: ...
    def WriteAs(self, value: int) -> CArchive: ...
    def WriteBuffer(self, Buffer: capsule, Count: int) -> None: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    @overload
    def WriteObject(self, o: CSerializable) -> None: ...
    @overload
    def WriteObject(constclassmrpt) -> void: ...
    def assign(self) -> CArchive: ...
    @overload
    def getArchiveDescription(self) -> str: ...
    @overload
    def getArchiveDescription(self) -> str: ...
    @overload
    def __lshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __lshift__(self, pObj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, obj: CSerializable) -> CArchive: ...
    @overload
    def __rshift__(self, pObj: CSerializable) -> CArchive: ...

class CExceptionEOF:
    @overload
    def __init__(self, s: str) -> None: ...
    @overload
    def __init__(self, arg0: CExceptionEOF) -> None: ...
    @overload
    def __init__(self, arg0: CExceptionEOF) -> None: ...
    def assign(self) -> CExceptionEOF: ...

class CSerializable(mrpt.pymrpt.mrpt.rtti.CObject):
    @overload
    def __init__(self, arg0: CSerializable) -> None: ...
    @overload
    def __init__(self) -> None: ...
    def GetRuntimeClass(self) -> mrpt.pymrpt.mrpt.rtti.TRuntimeClassId: ...
    def GetRuntimeClassIdStatic(self, *args, **kwargs) -> Any: ...
    def assign(self) -> CSerializable: ...

@overload
def ObjectToOctetVector(o: CSerializable, out_vector: List[int]) -> None: ...
@overload
def ObjectToOctetVector(constclassmrpt, classstd) -> void: ...
@overload
def OctetVectorToObject(in_data: List[int], obj: CSerializable) -> None: ...
@overload
def OctetVectorToObject(constclassstd, classstd) -> void: ...
@overload
def archiveFrom(s: mrpt.pymrpt.mrpt.io.CFileGZInputStream) -> CArchiveStreamBase_mrpt_io_CFileGZInputStream_t: ...
@overload
def archiveFrom(s: mrpt.pymrpt.mrpt.io.CFileGZOutputStream) -> CArchiveStreamBase_mrpt_io_CFileGZOutputStream_t: ...
@overload
def archiveFrom(s: mrpt.pymrpt.mrpt.io.CFileInputStream) -> CArchiveStreamBase_mrpt_io_CFileInputStream_t: ...
@overload
def archiveFrom(s: mrpt.pymrpt.mrpt.io.CFileOutputStream) -> CArchiveStreamBase_mrpt_io_CFileOutputStream_t: ...



from geometry_msgs.msg import Pose, PoseWithCovariance
from mrpt import pymrpt as pymrpt

def CPose3DQuat_to_ROS_Pose_msg(pq: _mrpt.poses.CPose3DQuat) -> Pose: ...
def ROS_Pose_msg_to_CPose3DQuat(p: Pose) -> _mrpt.poses.CPose3DQuat: ...
def CPose3D_to_ROS_Pose_msg(p: _mrpt.poses.CPose3D) -> Pose: ...
def ROS_Pose_msg_to_CPose3D(p: Pose) -> _mrpt.poses.CPose3D: ...
def CPose2D_to_ROS_Pose_msg(q: _mrpt.poses.CPose2D) -> Pose: ...
def ROS_Pose_msg_to_CPose2D(p: Pose) -> _mrpt.poses.CPose2D: ...
def ROS_PoseWithCovariance_msg_to_CPose3DPDFGaussian(p: PoseWithCovariance) -> _mrpt.poses.CPose3DPDFGaussian: ...
def CPose3DPDFGaussian_to_ROS_PoseWithCovariance_msg(p: _mrpt.poses.CPose3DPDFGaussian) -> PoseWithCovariance: ...



from typing import ClassVar

DETERMINING_ORIENTATION: nv_ins_status_type_t
INS_ALIGNING: nv_ins_status_type_t
INS_ALIGNMENT_COMPLETE: nv_ins_status_type_t
INS_HIGH_VARIANCE: nv_ins_status_type_t
INS_INACTIVE: nv_ins_status_type_t
INS_SOLUTION_FREE: nv_ins_status_type_t
INS_SOLUTION_GOOD: nv_ins_status_type_t
WAITING_INITIALPOS: nv_ins_status_type_t

class nv_ins_status_type_t:
    __doc__: ClassVar[str] = ...  # read-only
    __members__: ClassVar[dict] = ...  # read-only
    DETERMINING_ORIENTATION: ClassVar[nv_ins_status_type_t] = ...
    INS_ALIGNING: ClassVar[nv_ins_status_type_t] = ...
    INS_ALIGNMENT_COMPLETE: ClassVar[nv_ins_status_type_t] = ...
    INS_HIGH_VARIANCE: ClassVar[nv_ins_status_type_t] = ...
    INS_INACTIVE: ClassVar[nv_ins_status_type_t] = ...
    INS_SOLUTION_FREE: ClassVar[nv_ins_status_type_t] = ...
    INS_SOLUTION_GOOD: ClassVar[nv_ins_status_type_t] = ...
    WAITING_INITIALPOS: ClassVar[nv_ins_status_type_t] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __and__(self, other: object) -> object: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> object: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __or__(self, other: object) -> object: ...
    def __rand__(self, other: object) -> object: ...
    def __ror__(self, other: object) -> object: ...
    def __rxor__(self, other: object) -> object: ...
    def __setstate__(self, state: int) -> None: ...
    def __xor__(self, other: object) -> object: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

def enum2str(val: int) -> str: ...



from typing import Any, ClassVar

from typing import overload
import mrpt.pymrpt.mrpt.config
import mrpt.pymrpt.mrpt.math
import mrpt.pymrpt.mrpt.system
kfEKFAlaDavison: TKFMethod
kfEKFNaive: TKFMethod
kfIKF: TKFMethod
kfIKFFull: TKFMethod

class CKalmanFilterCapable_3UL_2UL_2UL_3UL_double_t:
    def __init__(self, *args, **kwargs) -> None: ...
    def OnNewLandmarkAddedToMap(self, in_obsIdx: int, in_idxNewFeat: int) -> None: ...
    @overload
    def OnNormalizeStateVector(self) -> None: ...
    @overload
    def OnNormalizeStateVector() -> void: ...
    @overload
    def OnPostIteration(self) -> None: ...
    @overload
    def OnPostIteration() -> void: ...
    def getLandmarkCov(self, idx: int, feat_cov: mrpt.pymrpt.mrpt.math.CMatrixFixed_double_2UL_2UL_t) -> None: ...
    def getLandmarkMean(self, idx: int, feat: mrpt.pymrpt.mrpt.math.CMatrixFixed_double_2UL_1UL_t) -> None: ...
    @overload
    def getNumberOfLandmarksInTheMap(self) -> int: ...
    @overload
    def getNumberOfLandmarksInTheMap() -> size_t: ...
    def getProfiler(self) -> mrpt.pymrpt.mrpt.system.CTimeLogger: ...
    @overload
    def getStateVectorLength(self) -> int: ...
    @overload
    def getStateVectorLength() -> size_t: ...
    def get_action_size(self, *args, **kwargs) -> Any: ...
    def get_feature_size(self, *args, **kwargs) -> Any: ...
    def get_observation_size(self, *args, **kwargs) -> Any: ...
    def get_vehicle_size(self, *args, **kwargs) -> Any: ...
    def internal_getPkk(self) -> mrpt.pymrpt.mrpt.math.CMatrixDynamic_double_t: ...
    def internal_getXkk(self, *args, **kwargs) -> Any: ...
    @overload
    def isMapEmpty(self) -> bool: ...
    @overload
    def isMapEmpty() -> bool: ...
    @property
    def KF_options(self) -> Any: ...

class CKalmanFilterCapable_7UL_3UL_3UL_7UL_double_t:
    def __init__(self, *args, **kwargs) -> None: ...
    def OnNewLandmarkAddedToMap(self, in_obsIdx: int, in_idxNewFeat: int) -> None: ...
    @overload
    def OnNormalizeStateVector(self) -> None: ...
    @overload
    def OnNormalizeStateVector() -> void: ...
    @overload
    def OnPostIteration(self) -> None: ...
    @overload
    def OnPostIteration() -> void: ...
    def getLandmarkCov(self, idx: int, feat_cov: mrpt.pymrpt.mrpt.math.CMatrixFixed_double_3UL_3UL_t) -> None: ...
    def getLandmarkMean(self, idx: int, feat: mrpt.pymrpt.mrpt.math.CMatrixFixed_double_3UL_1UL_t) -> None: ...
    @overload
    def getNumberOfLandmarksInTheMap(self) -> int: ...
    @overload
    def getNumberOfLandmarksInTheMap() -> size_t: ...
    def getProfiler(self) -> mrpt.pymrpt.mrpt.system.CTimeLogger: ...
    @overload
    def getStateVectorLength(self) -> int: ...
    @overload
    def getStateVectorLength() -> size_t: ...
    def get_action_size(self, *args, **kwargs) -> Any: ...
    def get_feature_size(self, *args, **kwargs) -> Any: ...
    def get_observation_size(self, *args, **kwargs) -> Any: ...
    def get_vehicle_size(self, *args, **kwargs) -> Any: ...
    def internal_getPkk(self) -> mrpt.pymrpt.mrpt.math.CMatrixDynamic_double_t: ...
    def internal_getXkk(self, *args, **kwargs) -> Any: ...
    @overload
    def isMapEmpty(self) -> bool: ...
    @overload
    def isMapEmpty() -> bool: ...
    @property
    def KF_options(self) -> Any: ...

class CParticleFilter:
    class TParticleFilterAlgorithm:
        __doc__: ClassVar[str] = ...  # read-only
        __members__: ClassVar[dict] = ...  # read-only
        __entries: ClassVar[dict] = ...
        pfAuxiliaryPFOptimal: ClassVar[CParticleFilter.TParticleFilterAlgorithm] = ...
        pfAuxiliaryPFStandard: ClassVar[CParticleFilter.TParticleFilterAlgorithm] = ...
        pfOptimalProposal: ClassVar[CParticleFilter.TParticleFilterAlgorithm] = ...
        pfStandardProposal: ClassVar[CParticleFilter.TParticleFilterAlgorithm] = ...
        def __init__(self, value: int) -> None: ...
        def __and__(self, other: object) -> object: ...
        def __eq__(self, other: object) -> bool: ...
        def __ge__(self, other: object) -> bool: ...
        def __getstate__(self) -> int: ...
        def __gt__(self, other: object) -> bool: ...
        def __hash__(self) -> int: ...
        def __index__(self) -> int: ...
        def __int__(self) -> int: ...
        def __invert__(self) -> object: ...
        def __le__(self, other: object) -> bool: ...
        def __lt__(self, other: object) -> bool: ...
        def __ne__(self, other: object) -> bool: ...
        def __or__(self, other: object) -> object: ...
        def __rand__(self, other: object) -> object: ...
        def __ror__(self, other: object) -> object: ...
        def __rxor__(self, other: object) -> object: ...
        def __setstate__(self, state: int) -> None: ...
        def __xor__(self, other: object) -> object: ...
        @property
        def name(self) -> str: ...
        @property
        def value(self) -> int: ...

    class TParticleFilterOptions(mrpt.pymrpt.mrpt.config.CLoadableOptions):
        BETA: float
        PF_algorithm: CParticleFilter.TParticleFilterAlgorithm
        adaptiveSampleSize: bool
        max_loglikelihood_dyn_range: float
        pfAuxFilterOptimal_MLE: bool
        pfAuxFilterOptimal_MaximumSearchSamples: int
        pfAuxFilterStandard_FirstStageWeightsMonteCarlo: bool
        powFactor: float
        resamplingMethod: CParticleFilter.TParticleResamplingAlgorithm
        sampleSize: int
        @overload
        def __init__(self) -> None: ...
        @overload
        def __init__(self, arg0: CParticleFilter.TParticleFilterOptions) -> None: ...
        @overload
        def __init__(self, arg0: CParticleFilter.TParticleFilterOptions) -> None: ...
        def assign(self) -> CParticleFilter.TParticleFilterOptions: ...
        @overload
        def loadFromConfigFile(self, source, section: str) -> None: ...
        @overload
        def loadFromConfigFile(constclassmrpt, conststd) -> void: ...
        @overload
        def saveToConfigFile(self, target, section: str) -> None: ...
        @overload
        def saveToConfigFile(classmrpt, conststd) -> void: ...

    class TParticleFilterStats:
        ESS_beforeResample: float
        weightsVariance_beforeResample: float
        def __init__(self) -> None: ...

    class TParticleResamplingAlgorithm:
        __doc__: ClassVar[str] = ...  # read-only
        __members__: ClassVar[dict] = ...  # read-only
        __entries: ClassVar[dict] = ...
        prMultinomial: ClassVar[CParticleFilter.TParticleResamplingAlgorithm] = ...
        prResidual: ClassVar[CParticleFilter.TParticleResamplingAlgorithm] = ...
        prStratified: ClassVar[CParticleFilter.TParticleResamplingAlgorithm] = ...
        prSystematic: ClassVar[CParticleFilter.TParticleResamplingAlgorithm] = ...
        def __init__(self, value: int) -> None: ...
        def __and__(self, other: object) -> object: ...
        def __eq__(self, other: object) -> bool: ...
        def __ge__(self, other: object) -> bool: ...
        def __getstate__(self) -> int: ...
        def __gt__(self, other: object) -> bool: ...
        def __hash__(self) -> int: ...
        def __index__(self) -> int: ...
        def __int__(self) -> int: ...
        def __invert__(self) -> object: ...
        def __le__(self, other: object) -> bool: ...
        def __lt__(self, other: object) -> bool: ...
        def __ne__(self, other: object) -> bool: ...
        def __or__(self, other: object) -> object: ...
        def __rand__(self, other: object) -> object: ...
        def __ror__(self, other: object) -> object: ...
        def __rxor__(self, other: object) -> object: ...
        def __setstate__(self, state: int) -> None: ...
        def __xor__(self, other: object) -> object: ...
        @property
        def name(self) -> str: ...
        @property
        def value(self) -> int: ...
    pfAuxiliaryPFOptimal: ClassVar[CParticleFilter.TParticleFilterAlgorithm] = ...
    pfAuxiliaryPFStandard: ClassVar[CParticleFilter.TParticleFilterAlgorithm] = ...
    pfOptimalProposal: ClassVar[CParticleFilter.TParticleFilterAlgorithm] = ...
    pfStandardProposal: ClassVar[CParticleFilter.TParticleFilterAlgorithm] = ...
    prMultinomial: ClassVar[CParticleFilter.TParticleResamplingAlgorithm] = ...
    prResidual: ClassVar[CParticleFilter.TParticleResamplingAlgorithm] = ...
    prStratified: ClassVar[CParticleFilter.TParticleResamplingAlgorithm] = ...
    prSystematic: ClassVar[CParticleFilter.TParticleResamplingAlgorithm] = ...
    m_options: Any
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilter) -> None: ...
    def assign(self) -> CParticleFilter: ...
    @overload
    def executeOn(self, obj, action, observation) -> None: ...
    @overload
    def executeOn(self, obj, action, observation, stats) -> None: ...

class CParticleFilterCapable:
    def __init__(self, *args, **kwargs) -> None: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS(EstimatedSampleSize) -> Any: ...
    @overload
    def ESS() -> double: ...
    def assign(self) -> CParticleFilterCapable: ...
    def defaultEvaluator(self, *args, **kwargs) -> Any: ...
    def fastDrawSample(self, *args, **kwargs) -> Any: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions) -> None: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions, out_particle_count: int) -> None: ...
    @overload
    def performResampling(conststructmrpt, size_t) -> void: ...
    def prediction_and_update(self, *args, **kwargs) -> Any: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...

class CParticleFilterDataImpl_mrpt_maps_CMultiMetricMapPDF_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_maps_CRBPFParticleData_mrpt_bayes_particle_storage_mode_POINTER_t(CParticleFilterCapable):
    @overload
    def __init__(self, arg0: CParticleFilterDataImpl_mrpt_maps_CMultiMetricMapPDF_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_maps_CRBPFParticleData_mrpt_bayes_particle_storage_mode_POINTER_t) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterDataImpl_mrpt_maps_CMultiMetricMapPDF_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_maps_CRBPFParticleData_mrpt_bayes_particle_storage_mode_POINTER_t) -> None: ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS() -> double: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS(EstimatedSampleSize) -> Any: ...
    @overload
    def ESS() -> double: ...
    @overload
    def assign(self) -> CParticleFilterDataImpl_mrpt_maps_CMultiMetricMapPDF_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_maps_CRBPFParticleData_mrpt_bayes_particle_storage_mode_POINTER_t: ...
    @overload
    def assign(self) -> CParticleFilterCapable: ...
    def defaultEvaluator(self, *args, **kwargs) -> Any: ...
    def derived(self, *args, **kwargs) -> Any: ...
    def fastDrawSample(self, *args, **kwargs) -> Any: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions) -> None: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions, out_particle_count: int) -> None: ...
    @overload
    def performResampling(conststructmrpt, size_t) -> void: ...
    def prediction_and_update(self, *args, **kwargs) -> Any: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...

class CParticleFilterDataImpl_mrpt_poses_CPointPDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t(CParticleFilterCapable):
    @overload
    def __init__(self, arg0: CParticleFilterDataImpl_mrpt_poses_CPointPDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterDataImpl_mrpt_poses_CPointPDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t) -> None: ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS() -> double: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS(EstimatedSampleSize) -> Any: ...
    @overload
    def ESS() -> double: ...
    @overload
    def assign(self) -> CParticleFilterDataImpl_mrpt_poses_CPointPDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t: ...
    @overload
    def assign(self) -> CParticleFilterCapable: ...
    def defaultEvaluator(self, *args, **kwargs) -> Any: ...
    def derived(self, *args, **kwargs) -> Any: ...
    def fastDrawSample(self, *args, **kwargs) -> Any: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions) -> None: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions, out_particle_count: int) -> None: ...
    @overload
    def performResampling(conststructmrpt, size_t) -> void: ...
    def prediction_and_update(self, *args, **kwargs) -> Any: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...

class CParticleFilterDataImpl_mrpt_poses_CPose3DPDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPose3D_mrpt_bayes_particle_storage_mode_VALUE_t(CParticleFilterCapable):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterDataImpl_mrpt_poses_CPose3DPDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPose3D_mrpt_bayes_particle_storage_mode_VALUE_t) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterDataImpl_mrpt_poses_CPose3DPDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPose3D_mrpt_bayes_particle_storage_mode_VALUE_t) -> None: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS() -> double: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS(EstimatedSampleSize) -> Any: ...
    @overload
    def ESS() -> double: ...
    @overload
    def assign(self) -> CParticleFilterDataImpl_mrpt_poses_CPose3DPDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPose3D_mrpt_bayes_particle_storage_mode_VALUE_t: ...
    @overload
    def assign(self) -> CParticleFilterCapable: ...
    def defaultEvaluator(self, *args, **kwargs) -> Any: ...
    def derived(self, *args, **kwargs) -> Any: ...
    def fastDrawSample(self, *args, **kwargs) -> Any: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions) -> None: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions, out_particle_count: int) -> None: ...
    @overload
    def performResampling(conststructmrpt, size_t) -> void: ...
    def prediction_and_update(self, *args, **kwargs) -> Any: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...

class CParticleFilterDataImpl_mrpt_poses_CPosePDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPose2D_mrpt_bayes_particle_storage_mode_VALUE_t(CParticleFilterCapable):
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterDataImpl_mrpt_poses_CPosePDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPose2D_mrpt_bayes_particle_storage_mode_VALUE_t) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterDataImpl_mrpt_poses_CPosePDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPose2D_mrpt_bayes_particle_storage_mode_VALUE_t) -> None: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS() -> double: ...
    @overload
    def ESS(self) -> float: ...
    @overload
    def ESS(EstimatedSampleSize) -> Any: ...
    @overload
    def ESS() -> double: ...
    @overload
    def assign(self) -> CParticleFilterDataImpl_mrpt_poses_CPosePDFParticles_std_deque_mrpt_bayes_CProbabilityParticle_mrpt_math_TPose2D_mrpt_bayes_particle_storage_mode_VALUE_t: ...
    @overload
    def assign(self) -> CParticleFilterCapable: ...
    def defaultEvaluator(self, *args, **kwargs) -> Any: ...
    def derived(self, *args, **kwargs) -> Any: ...
    def fastDrawSample(self, *args, **kwargs) -> Any: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def getW(self, i: int) -> float: ...
    @overload
    def getW(size_t) -> double: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def normalizeWeights(self) -> float: ...
    @overload
    def normalizeWeights(self, out_max_log_w: float) -> float: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def particlesCount(self) -> int: ...
    @overload
    def particlesCount() -> size_t: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions) -> None: ...
    @overload
    def performResampling(self, PF_options: CParticleFilter.TParticleFilterOptions, out_particle_count: int) -> None: ...
    @overload
    def performResampling(conststructmrpt, size_t) -> void: ...
    def prediction_and_update(self, *args, **kwargs) -> Any: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...
    @overload
    def setW(self, i: int, w: float) -> None: ...
    @overload
    def setW(size_t, double) -> void: ...

class CParticleFilterData_mrpt_maps_CRBPFParticleData_mrpt_bayes_particle_storage_mode_POINTER_t:
    m_particles: Any
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterData_mrpt_maps_CRBPFParticleData_mrpt_bayes_particle_storage_mode_POINTER_t) -> None: ...
    def assign(self) -> CParticleFilterData_mrpt_maps_CRBPFParticleData_mrpt_bayes_particle_storage_mode_POINTER_t: ...
    @overload
    def clearParticles(self) -> None: ...
    @overload
    def clearParticles() -> void: ...

class CParticleFilterData_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t:
    m_particles: Any
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterData_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t) -> None: ...
    def assign(self) -> CParticleFilterData_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t: ...
    @overload
    def clearParticles(self) -> None: ...
    @overload
    def clearParticles() -> void: ...
    def getMostLikelyParticle(self, *args, **kwargs) -> Any: ...

class CParticleFilterData_mrpt_math_TPose2D_mrpt_bayes_particle_storage_mode_VALUE_t:
    m_particles: Any
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterData_mrpt_math_TPose2D_mrpt_bayes_particle_storage_mode_VALUE_t) -> None: ...
    def assign(self) -> CParticleFilterData_mrpt_math_TPose2D_mrpt_bayes_particle_storage_mode_VALUE_t: ...
    @overload
    def clearParticles(self) -> None: ...
    @overload
    def clearParticles() -> void: ...
    def getMostLikelyParticle(self, *args, **kwargs) -> Any: ...

class CParticleFilterData_mrpt_math_TPose3D_mrpt_bayes_particle_storage_mode_VALUE_t:
    m_particles: Any
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CParticleFilterData_mrpt_math_TPose3D_mrpt_bayes_particle_storage_mode_VALUE_t) -> None: ...
    def assign(self) -> CParticleFilterData_mrpt_math_TPose3D_mrpt_bayes_particle_storage_mode_VALUE_t: ...
    @overload
    def clearParticles(self) -> None: ...
    @overload
    def clearParticles() -> void: ...
    def getMostLikelyParticle(self, *args, **kwargs) -> Any: ...

class CProbabilityParticleBase:
    log_w: float
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, logw: float) -> None: ...
    @overload
    def __init__(self, arg0: CProbabilityParticleBase) -> None: ...
    def assign(self) -> CProbabilityParticleBase: ...

class CProbabilityParticle_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t(CProbabilityParticleBase):
    d: Any
    log_w: float
    @overload
    def __init__(self, arg0: CProbabilityParticle_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t) -> None: ...
    @overload
    def __init__(self) -> None: ...
    @overload
    def assign(self) -> CProbabilityParticle_mrpt_math_TPoint3D_float_mrpt_bayes_particle_storage_mode_POINTER_t: ...
    @overload
    def assign(self) -> CProbabilityParticleBase: ...

class CProbabilityParticle_mrpt_math_TPose2D_mrpt_bayes_particle_storage_mode_VALUE_t(CProbabilityParticleBase):
    d: mrpt.pymrpt.mrpt.math.TPose2D
    log_w: float
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, data: mrpt.pymrpt.mrpt.math.TPose2D, logw: float) -> None: ...
    def assign(self) -> CProbabilityParticleBase: ...

class CProbabilityParticle_mrpt_math_TPose3D_mrpt_bayes_particle_storage_mode_VALUE_t(CProbabilityParticleBase):
    d: mrpt.pymrpt.mrpt.math.TPose3D
    log_w: float
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, data: mrpt.pymrpt.mrpt.math.TPose3D, logw: float) -> None: ...
    def assign(self) -> CProbabilityParticleBase: ...

class CRejectionSamplingCapable_mrpt_poses_CPose2D_mrpt_bayes_particle_storage_mode_POINTER_t:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: CRejectionSamplingCapable_mrpt_poses_CPose2D_mrpt_bayes_particle_storage_mode_POINTER_t) -> None: ...
    def assign(self) -> CRejectionSamplingCapable_mrpt_poses_CPose2D_mrpt_bayes_particle_storage_mode_POINTER_t: ...

class TKFMethod:
    __doc__: ClassVar[str] = ...  # read-only
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    kfEKFAlaDavison: ClassVar[TKFMethod] = ...
    kfEKFNaive: ClassVar[TKFMethod] = ...
    kfIKF: ClassVar[TKFMethod] = ...
    kfIKFFull: ClassVar[TKFMethod] = ...
    def __init__(self, value: int) -> None: ...
    def __and__(self, other: object) -> object: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> object: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __or__(self, other: object) -> object: ...
    def __rand__(self, other: object) -> object: ...
    def __ror__(self, other: object) -> object: ...
    def __rxor__(self, other: object) -> object: ...
    def __setstate__(self, state: int) -> None: ...
    def __xor__(self, other: object) -> object: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class TKF_options(mrpt.pymrpt.mrpt.config.CLoadableOptions):
    IKF_iterations: int
    debug_verify_analytic_jacobians: bool
    debug_verify_analytic_jacobians_threshold: float
    enable_profiler: bool
    method: TKFMethod
    use_analytic_observation_jacobian: bool
    use_analytic_transition_jacobian: bool
    @overload
    def __init__(self, verb_level_ref: mrpt.pymrpt.mrpt.system.VerbosityLevel) -> None: ...
    @overload
    def __init__(self, arg0: TKF_options) -> None: ...
    @overload
    def __init__(self, arg0: TKF_options) -> None: ...
    @overload
    def loadFromConfigFile(self, iniFile: mrpt.pymrpt.mrpt.config.CConfigFileBase, section: str) -> None: ...
    @overload
    def loadFromConfigFile(constclassmrpt, conststd) -> void: ...

class particle_storage_mode:
    __doc__: ClassVar[str] = ...  # read-only
    __members__: ClassVar[dict] = ...  # read-only
    POINTER: ClassVar[particle_storage_mode] = ...
    VALUE: ClassVar[particle_storage_mode] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    def __setstate__(self, state: int) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...



from typing import Any

from typing import overload

class CRandomGenerator:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, seed: int) -> None: ...
    def drawGaussian1D(self, mean: float, std: float) -> float: ...
    @overload
    def drawGaussian1D_normalized(self) -> float: ...
    @overload
    def drawGaussian1D_normalized() -> double: ...
    def drawUniform(self, Min: float, Max: float) -> float: ...
    @overload
    def drawUniform32bit(self) -> int: ...
    @overload
    def drawUniform32bit() -> uint32_t: ...
    @overload
    def drawUniform64bit(self) -> int: ...
    @overload
    def drawUniform64bit() -> uint64_t: ...
    @overload
    def randomize(self, seed: int) -> None: ...
    @overload
    def randomize(constunsignedint) -> void: ...
    @overload
    def randomize(self) -> None: ...
    @overload
    def randomize() -> void: ...

class Generator_MT19937:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: Generator_MT19937) -> None: ...
    def max(self, *args, **kwargs) -> Any: ...
    def min(self, *args, **kwargs) -> Any: ...
    @overload
    def seed(self, seed: int) -> None: ...
    @overload
    def seed(constunsignedint) -> void: ...
    def __call__(self) -> int: ...

@overload
def Randomize(seed: int) -> None: ...
@overload
def Randomize(constunsignedint) -> void: ...
@overload
def Randomize() -> None: ...
@overload
def Randomize() -> void: ...
def getRandomGenerator() -> CRandomGenerator: ...
@overload
def random_generator_for_STL(i: int) -> int: ...
@overload
def random_generator_for_STL(ptrdiff_t) -> ptrdiff_t: ...



from geometry_msgs.msg import Pose, PoseWithCovariance
from mrpt import pymrpt
import math

_mrpt = pymrpt.mrpt


def CPose3DQuat_to_ROS_Pose_msg(pq: _mrpt.poses.CPose3DQuat) -> Pose:
    """
    Converts from mrpt::poses::CPose3DQuat to ROS geometry_msgs.Pose
    """
    q = pq.asTPose()
    p = Pose()
    p.position.x = q.x
    p.position.y = q.y
    p.position.z = q.z
    p.orientation.x = q.qx
    p.orientation.y = q.qy
    p.orientation.z = q.qz
    p.orientation.w = q.qr
    return p


def ROS_Pose_msg_to_CPose3DQuat(p: Pose) -> _mrpt.poses.CPose3DQuat:
    """
    Converts to mrpt::poses::CPose3DQuat from ROS geometry_msgs.Pose
    """
    qq = _mrpt.math.TPose3DQuat(
        p.position.x, p.position.y, p.position.z,
        p.orientation.w, p.orientation.x, p.orientation.y, p.orientation.z)
    return _mrpt.poses.CPose3DQuat(qq)


def CPose3D_to_ROS_Pose_msg(p: _mrpt.poses.CPose3D) -> Pose:
    """
    Converts from mrpt::poses::CPose3D to ROS geometry_msgs.Pose
    """
    return CPose3DQuat_to_ROS_Pose_msg(_mrpt.poses.CPose3DQuat(p))


def ROS_Pose_msg_to_CPose3D(p: Pose) -> _mrpt.poses.CPose3D:
    """
    Converts to mrpt::poses::CPose3D from ROS geometry_msgs.Pose
    """
    return _mrpt.poses.CPose3D(ROS_Pose_msg_to_CPose3DQuat(p))


def CPose2D_to_ROS_Pose_msg(q: _mrpt.poses.CPose2D) -> Pose:
    """
    Converts from mrpt::poses::CPose2D to ROS geometry_msgs.Pose
    """
    p = Pose()
    p.position.x = q.x()
    p.position.y = q.y()
    p.position.z = 0.0
    p.orientation.x = 0.0
    p.orientation.y = 0.0
    p.orientation.z = 1.0 * math.sin(.5*q.phi())
    p.orientation.w = math.cos(.5*q.phi())
    return p


def ROS_Pose_msg_to_CPose2D(p: Pose) -> _mrpt.poses.CPose2D:
    """
    Converts to mrpt::poses::CPose2D from ROS geometry_msgs.Pose
    """
    return _mrpt.poses.CPose2D(ROS_Pose_msg_to_CPose3D(p))


#
# Read REP103: http://ros.org/reps/rep-0103.html#covariance-representation
#
# # Row-major representation of the 6x6 covariance matrix
# # The orientation parameters use a fixed-axis representation.
# # In order, the parameters are:
# # (x, y, z, rotation about X axis, rotation about Y axis, rotation about
# # Z axis)
# float64[36] covariance
#
# ==> MRPT uses non-fixed z-y-x and ROS uses fixed x-y-z rotations -->
# which
#     are equal except for the ordering --> only need to rearrange yaw,
#     pitch, roll

def ROS_PoseWithCovariance_msg_to_CPose3DPDFGaussian(p: PoseWithCovariance) -> _mrpt.poses.CPose3DPDFGaussian:
    """
    Converts to mrpt::poses::CPose3DPDFGaussian from ROS geometry_msgs.PoseWithCovariance
    """
    # rearrange covariance (see notes above)
    ind_map = [0, 1, 2, 5, 4, 3]  # X,Y,Z,YAW,PITCH,ROLL

    ret = _mrpt.poses.CPose3DPDFGaussian()
    ret.mean = ROS_Pose_msg_to_CPose3D(p.pose)

    for i in range(0, 6):
        for j in range(0, 6):
            ret.cov[i, j] = p.covariance[ind_map[i] * 6 + ind_map[j]]

    return ret


def CPose3DPDFGaussian_to_ROS_PoseWithCovariance_msg(p: _mrpt.poses.CPose3DPDFGaussian) -> PoseWithCovariance:
    """
    Converts from mrpt::poses::CPose3DPDFGaussian to ROS geometry_msgs.PoseWithCovariance
    """
    # rearrange covariance (see notes above)
    ind_map = [0, 1, 2, 5, 4, 3]  # X,Y,Z,YAW,PITCH,ROLL

    ret = PoseWithCovariance()
    ret.pose = CPose3D_to_ROS_Pose_msg(p.mean)

    for i in range(0, 6):
        for j in range(0, 6):
            ret.covariance[ind_map[i] * 6 + ind_map[j]] = p.cov[i, j]

    return ret
